/**
 * 
 * Author：黄钦潮 (Qinchao Huang)
 * Date: 2022/11/10
 * Mobile: 13085759937
 * QQ: 1412891143
 * Email：azzzzqq.com@qq.com
 * 
 */

 const gulp = require("gulp")
 const fs = require("fs")
 const autoprefixer = require("gulp-autoprefixer") // css添加前缀
 const sass = require("gulp-sass")(require("sass")) // 编译sass、scss
 const webserver = require("gulp-webserver") // 服务器
 const htmltpl = require('gulp-html-tpl')             // 引用html模板
 const artTemplate = require('art-template')        // 模板渲染
 // const imagemin = import('gulp-imagemin')            // 图片压缩
 
 let env = "";
 
 const config = {
     srcCSSPath: "./src/assets/css/*.css",
     srcSassPath: "./src/assets/scss/*.*ss",
     srcJSPath: "./src/assets/js/*",
     srcImgagePath: "./src/assets/images",
     srcHTMLPath: "./src/pages/*.html",
     outputPath: "./dist",
     defaultOpenPageFileName: "./index.html",
 }
 
 
 /**
  * 开发中中服务器
  */
 gulp.task("webHandler", function (done) {
     return gulp.src(config.outputPath)
         .pipe(webserver({
             port: '8882',
             livereload: true,
             directoryListing: true,
             open: config.defaultOpenPageFileName
         }))
 })
 
 /**
  * 监控src文件变化，会改变dist中的文件，造成刷新
  */
 gulp.task("watch", function () {
     gulp.watch(config.srcCSSPath, gulp.series("cssHandler"))
     gulp.watch(config.srcJSPath, gulp.series("jsHandler"))
     gulp.watch(config.srcSassPath, gulp.series("scssHandler"))
     gulp.watch(config.srcHTMLPath, gulp.series("htmlHandler"))
     gulp.watch(config.srcImgagePath, gulp.series("imagesHandler"))
 })
 
 
 /**
  * 
  * 图片处理
  * 
  */
 gulp.task('imagesHandler', function () {
     return gulp.src(config.srcImgagePath + "/*")
         .pipe(gulp.dest(config.outputPath + '/images'));
 });
 
 /**
  * 打包输出css
  */
 gulp.task("cssHandler", function (done) {
     return gulp.src(config.srcCSSPath)
         .pipe(autoprefixer({
             browsers: ['last 2 versions', 'Android >= 4.0'],
             cascade: true, //是否美化属性值 默认：true 像这样：
             //-webkit-transform: rotate(45deg);
             //        transform: rotate(45deg);
             remove: true //是否去掉不必要的前缀 默认：true
         }))
         .pipe(gulp.dest(config.outputPath + "/css"))
     done()
 })
 
 /**
  * 打包输出scss
  */
 gulp.task("scssHandler", function (done) {
     return gulp.src(config.srcSassPath)
         .pipe(sass())
         .pipe(autoprefixer(autoprefixerConfig()))
         .pipe(gulp.dest(config.outputPath + "/css"))
     done()
 })
 
 /**
  * 打包输出html
  */
 gulp.task("htmlHandler", function (done) {
     return gulp.src(config.srcHTMLPath)
         .pipe(htmltpl({
             tag: 'template',
             paths: ['./'],
             engine: function (template, data) {
                 return template && artTemplate.compile(template)(data);
             },
             data: {                // 初始化数据
             },
             beautify: {            // 美化html
                 "indent_size": "4",
                 "indent_char": " ",
                 "max_preserve_newlines": "-1",
                 "preserve_newlines": false,
             }
         }))
         .pipe(gulp.dest(config.outputPath))
     done()
 })
 
 /**
  * 打包输出js
  */
 gulp.task("jsHandler", function (done) {
     return gulp.src(config.srcJSPath)
         .pipe(gulp.dest(config.outputPath + "/js/"))
     done()
 })
 
 /**
  * 打包输出html
  */
 gulp.task("clean", function (done) {
     delfn(config.outputPath);
     done();
 })
 
 
 function delfn(p) {
     if (fs.existsSync(p)) {  //如果路径正确
         var list = fs.readdirSync(p)   //获取该目录下资源列表
         list.forEach((v, i) => {   //遍历该资源列表
             var path = p + '/' + v     //拼接新的路径
             var stats = fs.statSync(path)   //获取每一个资源的信息
             if (stats.isFile()) {  //如果是文件  删除
                 fs.unlinkSync(path)
             } else {//如果是目录  调用自己
                 delfn(path)
             }
         })
         fs.rmdirSync(p)  //删除空目录
     }
 }
 
 function autoprefixerConfig() {
     return {
         overrideBrowserslist: [
             "> 3% in CN",
             "Android >= 4.1",
             "iOS >= 7.1",
             "Chrome >= 31",
             "ff >= 31",
             "ie >= 8"
         ],
         grid: true
     }
 }
 
 
 gulp.task("setDev", function (done) {
     env = "dev";
     done()
 })
 
 gulp.task("setBuild", function (done) {
     env = "build";
     done()
 })
 
 
 gulp.task("dev", gulp.series(
     "setDev",
     "clean",
     gulp.parallel("scssHandler", "cssHandler", "jsHandler", "htmlHandler", "imagesHandler"),
     "webHandler",
     "watch"
 ))
 
 gulp.task("build", gulp.series(
     "setBuild",
     "clean",
     gulp.parallel("scssHandler", "cssHandler", "jsHandler", "htmlHandler", "imagesHandler"),
 ))