(function imgLazyLoad() {
    /**
     * 图片懒加载
     */
    //获取所有拥有data-src属性的img元素
    const lazyImages = document.querySelectorAll('img[data-src]');
    //定义img元素的src属性赋值方法，实现显示正确图片
    const imageLoad = function (image) {
        //将img元素的data-src属性值设置给src属性
        image.setAttribute('src', image.getAttribute('data-src'));
        //监听图片加载事件
        image.addEventListener('load', function () {
            //删除已加载img元素的data-src属性
            image.removeAttribute("data-src");
        })
    }
    if (IntersectionObserver) {
        //创建一个IntersectionObserver对象实例
        const intersectionObserver = new IntersectionObserver(function (images) {
            images.forEach(function (image) {
                //判断img元素是否可见，可见元素调用src属性赋值方法
                if (image.isIntersecting) {
                    //target属性为被观察的目标元素
                    imageLoad(image.target);
                    //调用IntersectionObserver的unobserve方法对完成懒加载的图片停止观察
                    intersectionObserver.unobserve(image.target);
                }
            });
        });
        //循环所有拥有data-src属性的img元素
        lazyImages.forEach(function (image) {
            //调用IntersectionObserver的observe方法进行观察
            intersectionObserver.observe(image);
        })
    } else {
        //循环所有拥有data-src属性的img元素
        lazyImages.forEach(function (image) {
            //调用IntersectionObserver的observe方法进行观察
            intersectionObserver.observe(image);
        })
    }
})()